####
#R code for simple Moving-habitat model 
#This file contains the code that was used to generate the simple Moving-habitat model example in the online case studies
#https://movinghabitatmodel.shinyapps.io/BasicModel/

 # The code is a companion to the following paper. Please refer to this paper for further details on the model
 #  Harsch, M.A, A. Phillips, Y. Zhou, M.-R. Leung, D.S. Rinnan, M. Kot. 
 #Moving forward: Insights and applications of moving-habitat models for climate change ecology. 
 #Journal of Ecology.
 
 #Code originally written by Melanie Harsch and Austin Phillips
 
 # The file contains code for both when xe (speed at which the habitat moves) is deterministic (constant between time steps) 
 #     and stochasitc (varies between time steps).
 # In both cases we provide:
 #      1. Fixed parameters and growth and dispersal functions functions
 #            Fixed parameters can be changed
 #            The growth function is set to the Beverton-Holt curve but can be easily changed
 #            We provide the Guassian, Laplace, and Cauchy dipsersal kernels. Use 'dispersal.type' to identify whether to use Gaussian, Laplace or Cauchy
 #      2. In the second step we get the population in equilibrium on the patch. 
 #                 If this is not done, changes in the population distribution may be due to the initial distribuiton and not the speed that the habitat moves
 #      3. We shift the habitat based on xe and apply the dispersal and growth functions. This is done 50 times (N) 
 #             At the end, we calculate final population size using the trapezoid rule
 
 
# Please install these libararies
library(graphics)
library(caTools)

################################################ 
## N: Number of iterations to run the simulation over
## D: Total length of fixed landscape - D depends on parameters and is defined below
## L: Habitat length
## n: Number of spatial subintervals over habitat. Number must be odd.
## dx: Length of spatial subinterval
## k: carrying capacity
## init_sigma: mean dispersal distance for establishing initial distribution
## xe: speed at which habitat moves at each time step
## R: Net reproductive rate

L<-5 
n<-201 
N<-50  
dx<- L/(n-1) 
k <- 100  
init_sigma<-2 
xe <- 1    
R <- 2    

# fvals: growth function applied to the population density within the habitat
# In this example, we use the Beverton-Holt curve. 
      fvals<-function() {
          fvals<- (R*patch_n0/(1 + (R-1)/k*patch_n0))
        }

#For plotting 
#create color gradation for lines in figures
greyshade <- seq(1,88,3)
#legend for text
leg.txt <- c("Initial population distribution", "Final population distribution")

################################################
## Define the dispersal function and relevant parameters
    ## One-dimensional dispersal kernel
    ## x,y: destination and source locations in one dimensional space
    ## dispersal.type: functional form of dispersal kernel
    ## param: controls variance of dispersal kernel
    
### Parameters
## dipersal.type: Reactive parameter defining functional form of dispersal kernel 
    # Can accomodate Gaussian, Laplace, or Cauchy 
## param: Reactive parameter defining variance of dispersal kernel

dispersal.type <- 'gaussian'   
param <- 1     
    
################ Dispersal function
    
  Kernel1 <- function(x,y,dispersal.type,param){
      if (dispersal.type=="gaussian"){
        sigma<-param
        return(1/(sqrt(2*pi)*sigma)*exp((-((x-y)^2))/(2*sigma^2)))
      }
      if(dispersal.type=="laplace"){
        alpha<-param
        return(alpha/2*exp(-alpha*abs(x-y)))
      }
      if(dispersal.type=="cauchy"){
        alpha<-param
        return(alpha/pi/(alpha^2 + (x-y)^2))
      }
    }
    
#####################################################################################
##Get population in equilibrium
########################################################################################
# number of iterations to run the burn-in to get pop in equilibrium with initial conditions before adding climate change)
burn <- 25 
## D: Define the total length of fixed landscape
## Length can be a fixed  number but equation ensures length is large enough to hold the entire population and the population will not crash after first iteration      
D<-xe*50+2*L 
## x: vector of positions over the entire landscape 
## init_patch_center: position of center of initial habitat
## y: vector of positions over the initial habitat
x<-seq(-D/2,D/2,dx)  
init_patch_center<- -D/2+1.2*L 
y<-seq(-L/2+init_patch_center, L/2+init_patch_center, dx) 
      
## left_index: gives the index of the vector x at which the habitat begins (from the left)
## right_index: gives the index of the vector x at which the habitat ends (from the left)
left_index<-(D/2-L/2+init_patch_center)/dx + 1
right_index<-left_index + L/dx
      
#initial distribution of the population within the patch. Uses Gaussian dispersal Kernel
      n_0 <- function(){
        n_0 <- 100/(sqrt(2*pi)*init_sigma)*exp(-(x-(init_patch_center+0))^2/(2*init_sigma^2))
      }
      
      n0 <- n_0()
      
      
#####################################################################################
#####################################################################################
# Begin Deterministic analysis
########################################################################################
########################################################################################

############################################################################
# Run the simulation for specified time steps to get population in equilibrium before allowing the patch to shift
  for(time in 1:burn){
        
## patch_n0: previous population densities that are within the habitat
      patch_n0<-n0[left_index:right_index]   
## K: matrix of kernel values for each (x,y) pair
      K<-    outer(x,y,"Kernel1",dispersal.type, param)    
        
## newn: new population density after the kernel is applied to fvals
    newn<-dx*K %*% fvals()
    n0 <- newn
    y<-x[left_index:right_index]
      }
      #########################################################################################
      #Plot initial distribution 
      par(mfrow=c(2,1))
      par(mar=c(6,4,2,2))
      plot(x,n0,type="l",xlab="Spatial gradient", ylab = "Population density", main="",xlim=c(-D/2,D/2),ylim=c(0,k+20), col="red", mgp=c(2,.5,0), cex.lab=1.3)
      legend("topright", leg.txt, lty=1, col=c("red","blue"), bty="n")
      mtext(expression(bold("Constant velocity of climate change")), side=3, adj=0, cex=1.3)
      
      
      #################################################################################
      #Run the simulation for N time steps, plotting distribution at end of each time step
        for (time in 2:N){
          
          patch_n0<-n0[left_index:right_index]
          K<-    outer(x,y,"Kernel1",dispersal.type, param)         
          newn<-dx*K %*% fvals()
          ## plot new population density newn in grey
          points(x,newn,type="l",col=paste("grey",greyshade[30-(time+1)]))
          
          #end simulation if population is -->0
          if(trapz(x,newn)<5){
            break   
          }
          ## If population is not approaching 0, continue simulation         
          ## assign the new population to be the current population
          n0<-newn
          ## shift habitat limits according to c_left and c_right
          left_index<-left_index+xe/dx
          right_index<-min(right_index+xe/dx,length(x))  
                    
          ## grab the new vector of habitat locations y
          ## if the habitat has shifted past the domain boundary on the right, only grab the relevant positions
          y<-x[left_index:right_index]

      #plot final distribution
      points((x),newn,type="l",col="blue", lwd=2)
      abline(h=0, lwd=2)
       }   #End simulation
       
      #calculate final population size using the trapezoid rule
      pop.size <-(trapz(y[1:2],newn[left_index:(left_index+1)]))*0.5+trapz(y[2:(length(y)-1)],newn[(left_index+1):(right_index-1)])+
        (trapz(y[1:2],newn[(right_index-1):right_index]))*0.5 
      
      if(trapz(x,newn)<5){
        mtext(paste("Population crashes after ", time, " time steps", sep=""), 1, line=3.5, cex=1.4)
      } else {
        mtext(paste("Population reaches ", round(pop.size), " after ", time, " time steps", sep=""), 1, line=3.5, cex=1.4)
      }
      
    
    
    
    #####################################################################################
    #####################################################################################
    # Begin stochastic analysis
    ########################################################################################
    ########################################################################################
#Note, some fixed parameters have changed from the deterministic analysis    
      #########
      D<-xe*50+2*L
      x<-seq(-D/2,D/2,dx)
      
      init_patch_center<- -D/2+1.2*L
      y<-seq(-L/2+init_patch_center, L/2+init_patch_center, dx)
      
      left_index<-(D/2-L/2+init_patch_center)/dx + 1
      right_index<-left_index + L/dx
      
      n_0 <- function()
      {
        n_0 <- 100/(sqrt(2*pi)*init_sigma)*exp(-(x-(init_patch_center+0))^2/(2*init_sigma^2))
      }
      
      n0 <- n_0()
      
      #run model for specified burn-in time steps to get population in equilibrium
      for(time in 1:burn){
        patch_n0<-n0[left_index:right_index]
        K<-    outer(x,y,"Kernel1",dispersal.type, param)    
                
        newn<-dx*K %*% fvals()
        n0 <- newn
        ## plot new population density newn in black
        y<-x[left_index:right_index]
      }
      
      ###############################
      # Plot initial population density

      par(mar=c(6,4,2,2))
      plot(x,n0,type="l",xlab="Spatial gradient", ylab = "Population density", main="",xlim=c(-D/2,D/2),ylim=c(-2,k+20), col="red",cex.lab=1.3,mgp=c(2,.5,0))
      legend("topright", leg.txt, lty=1, col=c("red","blue"), bty="n")
      mtext(expression(bold("Stochastic velocity of climate change")), side=3, adj=0, cex=1.3)
      
      
      ################################################ iteration of IDE
      # start with last iteration above and run for 50 time steps, plotting results of each time step
        for (time in 2:N){
          
          ## patch_n0: previous population densities that are within the habitat
          ## K: matrix of kernel values for each (x,y) pair
          ## fvals: growth function applied to the previous population density within the habitat
          ## newn: new population density after the kernel is applied to fvals
          patch_n0<-n0[left_index:right_index]
          K<-    outer(x,y,"Kernel1",dispersal.type, param)
                   
          newn<-dx*K %*% fvals()
          ## plot new population density newn in grey
          points(x,newn,type="l",col=paste("grey",greyshade[30-(time+1)]))
          
          # if population density -->0, end simulation
          if(trapz(x,newn)<5){
            break   
          }      
          
          ## assign the new population to be the old population
          ## shift habitat limits according to c_left and c_right
          ## grab the new vector of habitat locations y
          ## if the habitat has shifted past the domain boundary on the right, only grab the relevant positions
          n0<-newn
          shiftrate <- rnorm(1, xe, xe/2) #randomly draw the velocity of climate change
          left_index<-left_index+shiftrate/dx
          right_index<-min(right_index+shiftrate/dx,length(x))  
                  
          y<-x[left_index:right_index]
          
       #plot the final distribution
      points((x),newn,type="l",col="blue", lwd=2)
      abline(h=0, lwd=2)
      } 
      #population size at the end of the distribution using the trapezoid rule
      pop.size <-(trapz(y[1:2],newn[left_index:(left_index+1)]))*0.5+trapz(y[2:(length(y)-1)],newn[(left_index+1):(right_index-1)])+
        (trapz(y[1:2],newn[(right_index-1):right_index]))*0.5 
     
      if(trapz(x,newn)<5){
        mtext(paste("Population crashes after ", time, " time steps", sep=""), 1, line=3.5, cex=1.4)
      } else {
        mtext(paste("Population reaches ", round(pop.size), " after ", time, " time steps", sep=""), 1, line=3.5, cex=1.3)
      }
    
      
   
  ###########################################################################
  #End Model output
  #######################################################################
 